@@ -1,5 +1,6 @@
 setMethod("asJSON", "array", function(x, collapse = TRUE, na = NULL, oldna = NULL,
-  matrix = c("rowmajor", "columnmajor"), auto_unbox = FALSE, keep_vec_names = FALSE, ...) {
+  matrix = c("rowmajor", "columnmajor"), auto_unbox = FALSE, keep_vec_names = FALSE,
+  indent = NA_integer_, ...) {
 
   #validate
   matrix <- match.arg(matrix);
@@ -13,11 +14,11 @@ setMethod("asJSON", "array", function(x, collapse = TRUE, na = NULL, oldna = NUL
   # and therefore row major is required to match dimensions
   # dont pass auto_unbox (never unbox within matrix)
   margin <- ifelse(identical(matrix, "columnmajor") && isTRUE(collapse), length(dim(x)), 1);
-  tmp <- apply(x, margin, asJSON, matrix = matrix, na = na, ...)
+  tmp <- apply(x, margin, asJSON, matrix = matrix, na = na, indent = indent + 2L, ...)
 
   # collapse it
   if (collapse) {
-    collapse(tmp)
+    collapse(tmp, inner = FALSE, indent = indent)
   } else {
     tmp
   }
