@@ -144,7 +144,7 @@ stream_in <- function(con, handler, pagesize = 500, verbose = TRUE, ...) {
   i <- 1L;
   # JSON must be UTF-8 by spec
   while(length(page <- readLines(con, n = pagesize, encoding = "UTF-8"))){
-    if(verbose) message("Reading ", length(page), " lines (", i,").")
+    if(verbose) cat("\rFound", (i-1) * pagesize + length(page), "lines...")
     mydf <- simplify(lapply(page, parseJSON), ...);
     if(bind_pages){
       dfstack[[i]] <- mydf;
